#!/usr/bin/python

import gtksourceview2 as gsv
import gobject
import gtk

# chose not to use a dict here, because a list will
# allow us to use overlappting abbreviations
abbrevs = [ ('brb'  , 'be right back'),
            ('btw'  , 'between'),
            ('btw'  , 'by the way'),
            ('wrt'  , 'with regards to'),
            ('wrt'  , 'write'),
            ('lnx'  , 'Linux'),
            ('bus'  , 'business'),
          ]


def get_word(piter):
    """
        Borrowed from Jesse van den Kieboom and Ignacio Casal Quinteiro
        GPL v2 or greater.  Replace in the future.
    """
    if not piter.ends_word or piter.get_char() == '_':
        return None, None

    start = piter.copy()

    while True:
        if start.starts_line():
            break
        
        start.backward_char()
        ch = start.get_char()
        
        if not (ch.isalnum() or ch == '_' or ch == ':' or ch == '/'):
            start.forward_char()
            break

    if start.equal(piter):
        return None, None

    while (not start.equal(piter)) and start.get_char().isdigit():
        start.forward_char()

    if start.equal(piter):
        return None, None

    return start, start.get_text(piter)

class AbbreviationsProvider(gobject.GObject, gsv.CompletionProvider):
    
    
    def __init__(self, name, priority):
        gobject.GObject.__init__(self)
        self._name = name
        self._priority = priority
        self._proposals = []

    #def do_get_start_iter(self, context, proposal):
    #    buf = context.get_iter().get_buffer()
    #    mark = buf.get_mark(self.

    def do_match(self, context):
        return True

    def do_populate(self, context):
        start, word = get_word(context.get_iter())
        if not word: return None

        lw = len(word)

        theme = gtk.icon_theme_get_default()
        icon = theme.load_icon(gtk.STOCK_DIALOG_INFO,16,0)

        props=[]
        for a in abbrevs:
            if a[0][:lw] == word or a[1][:lw] == word:
                props.append(gsv.CompletionItem(a[1],a[1],icon,""))

        context.add_proposals(self, props, True )
       
    def do_get_name(self):
        return self._name

    def do_get_activation(self):
        return gsv.COMPLETION_ACTIVATION_INTERACTIVE #???

    def do_activate_proposal(self, proposal, piter):
        # ??? not sure what this does
        return False

    def do_get_piority(self):
        return self._priority

gobject.type_register(AbbreviationsProvider)

