#!/usr/bin/python

"""
    main app logic here

"""

import tempfile
import datetime

import gtk
import gobject
import glib
import pango
import gtksourceview2 as gsv

import completion
import speak

class MainWindow(gobject.GObject):

    def on_main_window_delete_event(self, window, somethingelse):
        # Wait for and clean up the child thread
        if self.speaker: self.speaker.join()
        # Tell GTK main loop to terminate
        gtk.main_quit()

    def on_edit_abbrevs_clicked(self, button):
        # populate the abbrevs_model

        self.abbrev_store.clear()
        for abbrev in completion.abbrevs:
            self.abbrev_store.append((abbrev[0], abbrev[1] ))

        response = self.abbrev_edit.run()
        if not response:
            # Revert was not clicked, so we can update the 
            # abbreviations dictionary with data from the 
            # list store.
            completion.abbrevs = []
            for row in self.abbrev_store:
                if row[0] != '<Type here>' and row[1] != '<Type here>':
                    completion.abbrevs.append( (row[0], row[1]) )

            # Here we could store the updated abbrevations list to disk
            # somewhere, or to GSettings on gtk 3.

        #otherwise, revert; do nothing, dictionary will not be updated
            
    def on_select_on_show_toggled(self, checkbutton):
        comp = self.view.get_completion()
        comp.set_property('select-on-show', checkbutton.get_active())

    def on_new_buffer_clicked(self, button):
        #if self.spoken_flag:
        #    _buffer = self.view.get_buffer()
        #    selection = (_buffer.get_start_iter(),
        #                 _buffer.get_end_iter())
        #    text = _buffer.get_text(selection[0],selection[1])
        #    if not text:
        #        return
        #    timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        #    self.history_store.append((timestamp,text))
   
        self.view.get_buffer().set_text('')
        self.view.grab_focus()

    def on_speak_button_clicked(self, button):
        _buffer = self.view.get_buffer()

        #get tuple of start, end iterators for selection, if any
        selection = _buffer.get_selection_bounds()

        if not selection:
            # we'll just grab the entire buffer
            selection = (_buffer.get_start_iter(),
                         _buffer.get_end_iter())

        # get the text itself based on the begin/end iterators
        text = _buffer.get_text(selection[0],selection[1])

        if not text:
            return

        f = tempfile.NamedTemporaryFile(delete=False)
        f.write(text)
        f.close()

        #if speach synthesis is still running, let's not start
        #another one
        if self.speaker:
            if self.speaker.running == True:
                return

            #otherwise reclaim the old speach synthesis thread as it
            #is done.
            self.speaker.join()

        print 'Saying: %s' % text
        self.speaker = speak.Speak(f.name)
        timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        self.history_store.append((timestamp,text))
        # here one could write the text out to a file for safe keeping
        # maybe with the format "YYYY-MM-DD HH:MM:SS.spoken.txt"
        # then when the program was restarted, it could read in stuff.

    def on_history_tree_row_activated(self, treeview, path, view_column):
        #print path

        # retrieve the text from the selected row using the path
        _iter = self.history_store.get_iter(path)
        text = self.history_store.get_value(_iter,1)

        # maybe could load text from file.  The history_store could 
        # store just a summary to save memory.
        self.view.get_buffer().set_text(text)

    def on_history_recall_clicked (self, button):
        tree_selection = self.history_tree.get_selection()
        rows = tree_selection.get_selected_rows()
        if not rows[1]: return
        #rows looks like this: ( <gtk.ListStore object>, [(0,)])
        _iter = rows[0].get_iter(rows[1][0])
        text = self.history_store.get_value(_iter,1)

        # maybe could load text from file.  The history_store could 
        # store just a summary to save memory.
        self.view.get_buffer().set_text(text)

    def on_history_remove_clicked(self, button):
        tree_selection = self.history_tree.get_selection()
        rows = tree_selection.get_selected_rows()
        if not rows[1]: return
        _iter = rows[0].get_iter(rows[1][0])
        self.history_store.remove(_iter)
        # could delete the corresponding file on disk too


    def on_abbrev_renderer_edited (self, cell, path, new_text):
        if not new_text == "<Type here":
            self.abbrev_store[path][0] =  new_text

    def on_abbrev_renderer_editing_started(self, cellrenderer, editable, path):
        _iter = self.abbrev_store.get_iter(path)
        if self.abbrev_store.get_value(_iter,0) == '<Type here>':
            self.abbrev_store.set_value(_iter,0,'')

    def on_word_renderer_edited (self, cell, path, new_text):
        self.abbrev_store[path][1] =  new_text
    
    def on_word_renderer_editing_started(self, cellrenderer, editable, path):
        _iter = self.abbrev_store.get_iter(path)
        if self.abbrev_store.get_value(_iter,1) == '<Type here>':
            self.abbrev_store.set_value(_iter,1,'')

    def on_abbrev_edit_delete_event(self, window, blah):
        self.abbrev_edit.hide()
        self.abbrev_edit.response(0)
        return False #do not actually delete the window!

    def on_close_button_clicked(self, button):
        self.abbrev_edit.hide()
        self.abbrev_edit.response(0)
       
    def on_revert_button_clicked(self, button):
        self.abbrev_edit.hide()
        self.abbrev_edit.response(1)

    def on_add_abbrev_button_clicked(self, button):
        self.abbrev_store.append(('<Type here>','<Type here>'))

    def on_remove_abbrev_button_clicked(self, button):
        tree_selection = self.abbrev_tree.get_selection()
        rows = tree_selection.get_selected_rows()
        #rows looks like this: ( <gtk.ListStore object>, [(0,)])
        if not rows[1]: return
        _iter = rows[0].get_iter(rows[1][0])
        rows[0].remove(_iter)
        

    def __init__(self):
        self.builder = gtk.Builder()
        self.builder.add_from_file("tts_assist.glade")


        # main window
        self.editor = self.builder.get_object('editor')
        self.history_tree = self.builder.get_object('history_tree')
        self.speak_button = self.builder.get_object('speak_button')
        self.new_buffer   = self.builder.get_object('new_buffer')
        
        self.select_on_show = self.builder.get_object('select_on_show')
        self.edit_abbrevs   = self.builder.get_object('edit_abbrevs')

        self.main_window   = self.builder.get_object('main_window')

        self.history_tree = self.builder.get_object('history_tree')
        self.history_store = self.builder.get_object('history_store')

        self.history_tree.set_headers_visible(True)

        # edit dialog
        self.abbrev_edit = self.builder.get_object('abbrev_edit')
        self.abbrev_edit.resize(400,300)

        self.abbrev_store = self.builder.get_object('abbrev_store')
        self.abbrev_tree = self.builder.get_object('abbrev_tree')

        # Create GtkSourceView buffer
        self._buffer = gsv.Buffer()
        
        # and add it to a GtkSourceView View
        self.view = gsv.View(self._buffer)
        self.view.set_wrap_mode(gtk.WRAP_WORD)
        self.view.show()

        pangoFont = pango.FontDescription("Sans 16")
        self.view.modify_font(pangoFont)

        self.editor.add(self.view)
        self.view.grab_focus()

        # Tell the View about the completions (abbreviations) we
        # want to use.
        comp = self.view.get_completion()
        # set the gui to match the completion object's existing state
        self.select_on_show.set_property("active", 
                                         comp.get_property("select-on-show"))
        
        # This is a built-in provider that suggests words the user has
        # already typed.  It might be more trouble than it's worth
        words_provider = gsv.CompletionWords()
        words_provider.register(self.view.get_buffer())

        # Here's our abbreviations provider
        abbrevs_provider = completion.AbbreviationsProvider('Abbreviations', 0)

        # Actually activate them
        comp.add_provider(abbrevs_provider)
        comp.add_provider(words_provider)

        self.main_window.resize(800,400)

        # Connect all signals that we defined in glade-3.
        self.builder.connect_signals(self)

        # instance var to hold speaker thread
        self.speaker = None

        # Here one could populate the history list store with filenames and 
        # content from disk.  Commented out because current there is no code
        # to create these files

        """
        import glob

        filelist = glob.glob("*.spoken.txt")
        for f in filelist:
            _file = fopen(f,'r')
            date = f.split('.')[0]
            with _file:
                self.history_store.append((date, _file.read()))
        """

        # we could also read abbrevations from a config file perhaps
        # with gtk 3, we would use GSettings to do this in a consistant way
        # with gtk 2, we have to do it ourselves

        self.main_window.show()


if __name__ == '__main__':
    glib.threads_init()

    a = MainWindow()
    gtk.main()
